@@ -1,54 +1,24 @@
-//! This crate implements a type that is a thin wrapper around uuids that
-//! can hold a "debug id".  This is a concept that originally comes from
-//! breakpad and is also used by Sentry to identify a debug information
-//! file.
+//! This crate provides types for identifiers of object files, such as executables, dynamic
+//! libraries or debug companion files. The concept originates in Google Breakpad and defines two
+//! types:
 //!
-//! The reason this is not just a UUID is that at least on Windows a debug
-//! information file (PDB) is not just associated by using a UUID alone
-//! but it has an appendix (a `u32` age field).
+//!  - [`CodeId`]: Identifies the file containing source code, i.e. the actual library or
+//!    executable. The identifier is platform dependent and implementation defined. Thus, there is
+//!    no canonical representation.
+//!  - [`DebugId`]: Identifies a debug information file, which may or may not use information from
+//!    the Code ID. The contents are also implementation defined, but as opposed to `CodeId`, the
+//!    structure is streamlined across platforms. It is also guaranteed to be 32 bytes in size.
 //!
-//! ## Representation
-//!
-//! The string representation must be between 33 and 40 characters long and
-//! consist of:
-//!
-//! 1. 36 character hyphenated hex representation of the UUID field
-//! 2. 1-16 character lowercase hex representation of the u64 appendix
-//!
-//! ```
-//! # extern crate debugid;
-//! use debugid::DebugId;
-//!
-//! # fn foo() -> Result<(), ::debugid::ParseDebugIdError> {
-//! let id: DebugId = "dfb8e43a-f242-3d73-a453-aeb6a777ef75-a".parse()?;
-//! assert_eq!("dfb8e43a-f242-3d73-a453-aeb6a777ef75-a".to_string(), id.to_string());
-//! # Ok(())
-//! # }
-//!
-//! # fn main() { foo().unwrap() }
-//! ```
-//!
-//! ## Breakpad compatibility
-//!
-//! Separately the breakpad format can be generated as well:
-//!
-//! ```
-//! # extern crate debugid;
-//! use debugid::DebugId;
-//!
-//! # fn foo() -> Result<(), ::debugid::ParseDebugIdError> {
-//! let id: DebugId = "dfb8e43a-f242-3d73-a453-aeb6a777ef75-a".parse()?;
-//! assert_eq!(id.breakpad().to_string(), "DFB8E43AF2423D73A453AEB6A777EF75a");
-//! # Ok(())
-//! # }
-//!
-//! # fn main() { foo().unwrap() }
-//! ```
+//! [`CodeId`]: struct.CodeId.html [`DebugId`]: struct.DebugId.html
+
+#![warn(missing_docs)]
 
-use regex::Regex;
 use std::error;
 use std::fmt;
+use std::fmt::Write;
 use std::str;
+
+use regex::Regex;
 use uuid::Uuid;
 
 lazy_static::lazy_static! {
@@ -75,8 +45,8 @@ lazy_static::lazy_static! {
     .unwrap();
 }
 
-/// Indicates a parsing error
-#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+/// Indicates an error parsing a [`DebugId`](struct.DebugId.html).
+#[derive(Clone, Copy, Debug, Eq, PartialEq)]
 pub struct ParseDebugIdError;
 
 impl error::Error for ParseDebugIdError {
@@ -100,7 +70,10 @@ impl fmt::Display for ParseDebugIdError {
 /// 1. 36 character hyphenated hex representation of the UUID field
 /// 2. 1-16 character lowercase hex representation of the u64 appendix
 ///
-/// **Example:**
+/// The debug identifier is compatible to Google Breakpad. Use [`DebugId::breakpad`] to get a
+/// breakpad string representation of this debug identifier.
+///
+/// # Example
 ///
 /// ```
 /// # extern crate debugid;
@@ -124,13 +97,18 @@ pub struct DebugId {
 }
 
 impl DebugId {
+    /// Constructs an empty debug identifier, containing only zeros.
+    pub fn nil() -> Self {
+        Self::default()
+    }
+
     /// Constructs a `DebugId` from its `uuid`.
-    pub fn from_uuid(uuid: Uuid) -> DebugId {
+    pub fn from_uuid(uuid: Uuid) -> Self {
         Self::from_parts(uuid, 0)
     }
 
     /// Constructs a `DebugId` from its `uuid` and `appendix` parts.
-    pub fn from_parts(uuid: Uuid, appendix: u32) -> DebugId {
+    pub fn from_parts(uuid: Uuid, appendix: u32) -> Self {
         DebugId {
             uuid,
             appendix,
@@ -139,7 +117,7 @@ impl DebugId {
     }
 
     /// Constructs a `DebugId` from a Microsoft little-endian GUID and age.
-    pub fn from_guid_age(guid: &[u8], age: u32) -> Result<DebugId, ParseDebugIdError> {
+    pub fn from_guid_age(guid: &[u8], age: u32) -> Result<Self, ParseDebugIdError> {
         if guid.len() != 16 {
             return Err(ParseDebugIdError);
         }
@@ -153,7 +131,7 @@ impl DebugId {
     }
 
     /// Parses a breakpad identifier from a string.
-    pub fn from_breakpad(string: &str) -> Result<DebugId, ParseDebugIdError> {
+    pub fn from_breakpad(string: &str) -> Result<Self, ParseDebugIdError> {
         // Technically, we are are too permissive here by allowing dashes, but
         // we are complete.
         string.parse()
@@ -172,6 +150,11 @@ impl DebugId {
         self.appendix
     }
 
+    /// Returns whether this identifier is nil, i.e. it consists only of zeros.
+    pub fn is_nil(&self) -> bool {
+        self.uuid.is_nil() && self.appendix() == 0
+    }
+
     /// Returns a wrapper which when formatted via `fmt::Display` will format a
     /// a breakpad identifier.
     pub fn breakpad(&self) -> BreakpadFormat<'_> {
@@ -230,42 +213,6 @@ impl From<(Uuid, u32)> for DebugId {
     }
 }
 
-#[cfg(feature = "with_serde")]
-mod serde_support {
-    use serde::de::{self, Deserialize, Deserializer, Unexpected, Visitor};
-    use serde::ser::{Serialize, Serializer};
-
-    use super::*;
-
-    impl<'de> Deserialize<'de> for DebugId {
-        fn deserialize<D: Deserializer<'de>>(deserializer: D) -> Result<Self, D::Error> {
-            struct V;
-
-            impl<'de> Visitor<'de> for V {
-                type Value = DebugId;
-
-                fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
-                    formatter.write_str("DebugId")
-                }
-
-                fn visit_str<E: de::Error>(self, value: &str) -> Result<DebugId, E> {
-                    value
-                        .parse()
-                        .map_err(|_| de::Error::invalid_value(Unexpected::Str(value), &self))
-                }
-            }
-
-            deserializer.deserialize_str(V)
-        }
-    }
-
-    impl Serialize for DebugId {
-        fn serialize<S: Serializer>(&self, serializer: S) -> Result<S::Ok, S::Error> {
-            serializer.serialize_str(&self.to_string())
-        }
-    }
-}
-
 /// Wrapper around `DebugId` for Breakpad formatting.
 ///
 /// **Example:**
@@ -298,3 +245,125 @@ impl<'a> fmt::Display for BreakpadFormat<'a> {
         )
     }
 }
+
+/// Indicates an error parsing a [`CodeId`](struct.CodeId.html).
+#[derive(Clone, Copy, Debug, Eq, PartialEq)]
+pub struct ParseCodeIdError;
+
+/// Unique platform-dependent identifier of code files.
+///
+/// This identifier assumes a string representation that depends on the platform and compiler used.
+/// There are the following known formats:
+///
+///  - **MachO UUID**: The unique identifier of a Mach binary, specified in the `LC_UUID` load
+///    command header.
+///  - **GNU Build ID**: Contents of the `.gnu.build-id` note or section contents formatted as
+///    lowercase hex string.
+///  - **PE Timestamp**: Timestamp and size of image values from a Windows PE header.
+///  - **GO Build ID**: Nested GO build identifiers comprising `actionID/[.../]contentID`.
+#[derive(Clone, Default, Eq, Hash, Ord, PartialEq, PartialOrd)]
+pub struct CodeId {
+    inner: String,
+}
+
+impl CodeId {
+    /// Constructs an empty code identifier.
+    pub fn nil() -> Self {
+        Self::default()
+    }
+
+    /// Constructs a `CodeId` from its string representation.
+    pub fn new(string: String) -> Self {
+        CodeId { inner: string }
+    }
+
+    /// Constructs a `CodeId` from a binary slice.
+    pub fn from_binary(slice: &[u8]) -> Self {
+        let mut string = String::with_capacity(slice.len() * 2);
+
+        for byte in slice {
+            write!(&mut string, "{:02x}", byte).expect("");
+        }
+
+        Self::new(string)
+    }
+
+    /// Returns whether this identifier is nil, i.e. it is empty.
+    pub fn is_nil(&self) -> bool {
+        self.inner.is_empty()
+    }
+
+    /// Returns the string representation of this code identifier.
+    pub fn as_str(&self) -> &str {
+        self.inner.as_str()
+    }
+}
+
+impl fmt::Display for CodeId {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.write_str(&self.inner)
+    }
+}
+
+impl fmt::Debug for CodeId {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_tuple("CodeId").field(&self.as_str()).finish()
+    }
+}
+
+impl str::FromStr for CodeId {
+    type Err = ParseCodeIdError;
+
+    fn from_str(string: &str) -> Result<Self, ParseCodeIdError> {
+        Ok(Self::new(string.into()))
+    }
+}
+
+#[cfg(feature = "serde")]
+mod serde_support {
+    use serde::de::{self, Deserialize, Deserializer, Unexpected, Visitor};
+    use serde::ser::{Serialize, Serializer};
+
+    use super::*;
+
+    impl Serialize for CodeId {
+        fn serialize<S: Serializer>(&self, serializer: S) -> Result<S::Ok, S::Error> {
+            serializer.serialize_str(self.as_str())
+        }
+    }
+
+    impl<'de> Deserialize<'de> for CodeId {
+        fn deserialize<D: Deserializer<'de>>(deserializer: D) -> Result<Self, D::Error> {
+            let string = String::deserialize(deserializer)?;
+            Ok(CodeId::new(string))
+        }
+    }
+
+    impl<'de> Deserialize<'de> for DebugId {
+        fn deserialize<D: Deserializer<'de>>(deserializer: D) -> Result<Self, D::Error> {
+            struct V;
+
+            impl<'de> Visitor<'de> for V {
+                type Value = DebugId;
+
+                fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
+                    formatter.write_str("DebugId")
+                }
+
+                fn visit_str<E: de::Error>(self, value: &str) -> Result<DebugId, E> {
+                    value
+                        .parse()
+                        .map_err(|_| de::Error::invalid_value(Unexpected::Str(value), &self))
+                }
+            }
+
+            deserializer.deserialize_str(V)
+        }
+    }
+
+    impl Serialize for DebugId {
+        fn serialize<S: Serializer>(&self, serializer: S) -> Result<S::Ok, S::Error> {
+            serializer.serialize_str(&self.to_string())
+        }
+    }
+}
